# -*- coding: utf-8 -*-
"""
:Author: HuangJianYi
:Date: 2024-01-18 18:19:58
@LastEditTime: 2025-05-28 19:46:16
@LastEditors: HuangJianYi
:Description: 框架DB操作类
"""
from seven_framework.base_model import *
from seven_framework import *
from seven_cloudapp_frame.libs.common import *
from seven_cloudapp_frame.libs.customize.seven_helper import *


class FrameDbModel(BaseModel):

    def __init__(self, model_class, sub_table):
        """
        :Description: 框架DB操作类
        :param model_class: 实体对象类
        :param sub_table: 分表标识
        :last_editors: HuangJianYi
        """
        super(FrameDbModel,self).__init__(model_class, sub_table)

    def add_list_batch(self, data_list, batch_num=100):
        """
        :description: 分批添加数据
        :param data_list：数据列表
        :param batch_num：分批数量
        :return 成功添加的数量
        :last_editors: HuangJianYi
        """
        page_index = 0
        total = 0
        while True:
            add_list = data_list[page_index * batch_num:(page_index + 1) * batch_num]
            if not add_list:
                return total
            result = self.add_list(add_list)
            if result == True:
                total += len(add_list)
            page_index += 1

    def set_sub_table(self, object_id=''):
        """
        :description: 设置分表
        :param object_id:object_id
        :return:
        :last_editors: HuangJianYi
        """
        table_name = str(self.model_obj).lower()
        sub_table_config = share_config.get_value("sub_table_config",{})
        table_config = sub_table_config.get(table_name, None)
        if table_config and object_id:
            calculate_type = sub_table_config.get("calculate_type", 1) # 1-默认 2-去掉最后1位再取模（淘宝的应用标识尾数只有1-5会导致分布不均）
            if calculate_type == 2:
                from seven_cloudapp_frame.models.enum import PlatType
                calculate_value = sub_table_config.get("calculate_value")
                plat_type = share_config.get_value("plat_type", PlatType.tb.value)  # 平台类型
                if calculate_value and plat_type == PlatType.tb.value:
                    first_value = int(object_id[0:11]) # 取前11位
                    if first_value >= int(calculate_value):
                        object_id = object_id[:-1]
                else:
                    object_id = object_id[:-1]

            sub_table = SevenHelper.get_sub_table(object_id, table_config.get("sub_count", 10))
            if sub_table:
                # 数据库表名
                self.table_name = table_name.replace("_tb", f"_{sub_table}_tb")
        return self

    def set_view(self, view_name=''):
        """
        :description: 设置视图
        :param view_name:视图名
        :return:
        :last_editors: HuangJianYi
        """
        table_name = str(self.model_obj).lower()
        if not view_name:
            self.table_name = table_name.replace("_tb", "_view")
        else:
            self.table_name = view_name
        return self

    def relation_and_merge_dict_list(self, primary_dict_list, relation_db_model, relation_key_field, field="*", primary_key_field="id", is_cache=True, dependency_key="", cache_expire=1800):
        """
        :description: 根据给定的主键表关联ID数组从关联表获取字典列表合并。
        :param primary_dict_list: 主表字典列表
        :param relation_db_model: 关联表关联model
        :param relation_key_field:  关联表关联字段
        :param field: 关联表查询字段
        :param primary_key_field: 主表关联字段
        :param is_cache: 是否开启缓存（1-是 0-否）
        :param dependency_key: 缓存依赖键
        :param cache_expire: 缓存过期时间（秒）
        :return:
        :last_editors: HuangJianYi
        """
        if len(primary_dict_list) <= 0:
            return primary_dict_list
        # 检查relation_key_field是否已经在field中
        if field != "*" and relation_key_field not in field.split(","):
            field = f"{relation_key_field},{field}"
        primary_key_field_type = 1  # 1-整形 2-字符串
        ext_table_ids = [i[primary_key_field] for i in primary_dict_list]
        if len(ext_table_ids) > 0:
            if isinstance(ext_table_ids[0], str):
                primary_key_field_type = 2
        if primary_key_field_type == 1:
            where = SevenHelper.get_condition_by_int_list(relation_key_field, ext_table_ids)
        else:
            where = SevenHelper.get_condition_by_str_list(relation_key_field, ext_table_ids)
        if is_cache == True:
            relation_dict_list = relation_db_model.get_cache_dict_list(where, field=field, dependency_key=dependency_key, cache_expire=cache_expire)
        else:
            relation_dict_list = relation_db_model.get_dict_list(where, field=field)
        dict_list = SevenHelper.merge_dict_list(primary_dict_list, primary_key_field, relation_dict_list, relation_key_field, exclude_merge_columns_names="id")
        return dict_list

    def relation_and_merge_dict(self, primary_dict, relation_db_model, field="*", primary_key_field="id", is_cache=True, dependency_key="", cache_expire=1800):
        """
        :description: 根据给定的主键表字典合并关联表字典。
        :param primary_dict: 主表字典
        :param relation_db_model: 关联表关联model
        :param field: 关联表查询字段
        :param primary_key_field: 主表关联字段
        :param is_cache: 是否开启缓存（1-是 0-否）
        :param dependency_key: 缓存依赖键
        :param cache_expire: 缓存过期时间（秒）
        :return:
        :last_editors: HuangJianYi
        """
        if not primary_dict:
            return primary_dict
        if is_cache == True:
            relation_dict = relation_db_model.get_cache_dict_by_id(primary_dict[primary_key_field], field=field, dependency_key=dependency_key, cache_expire=cache_expire)
        else:
            relation_dict = relation_db_model.get_dict_by_id(primary_dict[primary_key_field], field=field)
        if relation_dict and "id" in relation_dict:
            del relation_dict["id"]
        if relation_dict:
            primary_dict.update(relation_dict)
        return primary_dict

    def call_procedure(self, procedure_name: str, params: list = None):
        """
        :Description: 调用存储过程
        :param procedure_name: 存储过程名称
        :param params: 参数值(数组)
        :return: 存储过程的结果
        """
        self.db.connection()
        try:
            with self.db._conn.cursor(pymysql.cursors.DictCursor) as cursor:
                cursor.callproc(procedure_name, params)
                result = cursor.fetchall()
            self.db._conn.commit()
            return result
        except Exception as e:
            self.db._conn.rollback()
            print(traceback.print_exc())
            raise Exception("mysql execute error:" + str(e) + " procedure:" +
                            str(procedure_name) + " 参数:" + str(params))
        finally:
            self.db._conn.close()

    def convert_db_config(self, db_connect_key, is_auto=False):
        """
        :Description: 转换最终的连接串配置
        :param db_connect_key: db_connect_key
        :param is_auto: True-有主从配置，根据语句自动分配读写库
        :return: 
        """
        if isinstance(db_connect_key, str):
            db_config = config.get_value(db_connect_key)
        else:
            db_config = db_connect_key
        if not db_config:
            return db_config
        covert_config = {}
        if is_auto:
            covert_config["host"] = db_config.get("auto_host", db_config.get("host", ""))
            covert_config["port"] = db_config.get("auto_port", db_config.get("port", ""))
            covert_config["user"] = db_config.get("auto_user", db_config.get("user", ""))
            covert_config["passwd"] = db_config.get("auto_password", db_config.get("passwd", ""))
        else:
            covert_config["host"] = db_config.get("host", "")
            covert_config["port"] = db_config.get("port", "")
            covert_config["user"] = db_config.get("user", "")
            covert_config["passwd"] = db_config.get("passwd", "")

        covert_config["db"] = db_config.get("db", "")
        covert_config["charset"] = db_config.get("charset", "")
        return covert_config

    def add_values(self, model_list, ignore=False, is_doris=False, update_feild_list=None, exclude_update_feild_list=None):
        """
        :description: 一次性数据写入(insert into... values(...),(...),(...);)
        :param model_list: 数据模型列表
        :param ignore: 忽略已存在的记录
        :param is_doris: 是否doris
        :param update_feild_list: 触发唯一键时需要更新的字段列表(doris=True)
        :param exclude_update_feild_list: 触发唯一键时需排除更新的字段列表(doris=True)
        :return 成功True 失败False
        :last_editors: HuangJingCan
        """
        if not model_list or len(model_list) == 0:
            return False

        field_list = self.model_obj.get_field_list()
        if len(field_list) == 0:
            return False

        if is_doris is True:
            if update_feild_list:
                field_list = [field for field in field_list if field in update_feild_list]
            elif exclude_update_feild_list:
                field_list = [field for field in field_list if field not in exclude_update_feild_list]

        insert_field_str = ""
        for field_str in field_list:
            if str(field_str).lower() == self.primary_key_field.lower() and not is_doris:
                continue
            insert_field_str += str(f"`{field_str}`,")

        insert_field_str = insert_field_str.rstrip(',')
        doris_sql = "set enable_unique_key_partial_update=true; set enable_insert_strict=false;" if is_doris else ""
        sql = f"{doris_sql} INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUES "
        param = []

        for model in model_list:
            insert_value_str = ""
            for field_str in field_list:
                param_value = str(getattr(model, field_str))
                if str(field_str).lower() == self.primary_key_field.lower() and not is_doris:
                    continue
                insert_value_str += "%s,"
                param.append(param_value)

            insert_value_str = insert_value_str.rstrip(',')
            sql += f"({insert_value_str}),"

        sql = sql.rstrip(',') + ";"

        if self.is_transaction():
            sql_item = {}
            sql_item["sql"] = sql
            sql_item["params"] = tuple(param)
            self.db_transaction.transaction_list.append(sql_item)
        else:
            self.db.insert(sql, tuple(param), False)
        return True



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-11-01 18:56:43
@LastEditTime: 2024-01-03 10:45:51
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class UserAddressModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(UserAddressModel, self).__init__(UserAddress, sub_table)
        db_connect_key, self.redis_config_dict = SevenHelper.get_connect_config("db_user","redis_user", db_connect_key)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类

class UserAddress:

    def __init__(self):
        super(UserAddress, self).__init__()
        self.id = 0  # id
        self.app_id = ""  # 应用标识
        self.act_id = 0  # 活动标识
        self.user_id = 0  # 用户标识
        self.open_id = ""  # open_id
        self.real_name = ""  # 真实姓名
        self.telephone = ""  # 手机号码
        self.is_default = 0  # 是否默认地址（1是0否）
        self.province = ""  # 省
        self.city = ""  # 市
        self.county = ""  # 区
        self.street = ""  # 所在街道
        self.address = ""  # 收货地址
        self.create_date = "1900-01-01 00:00:00"  # 创建时间
        self.remark = ""  # 备注

    @classmethod
    def get_field_list(self):
        return ['id', 'app_id', 'act_id', 'user_id', 'open_id', 'real_name', 'telephone', 'is_default', 'province', 'city', 'county', 'street', 'address', 'create_date', 'remark']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "user_address_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-10-22 13:32:07
@LastEditTime: 2025-05-12 10:08:35
@LastEditors: HuangJianYi
@Description: 风险控制帮助类
"""
from seven_framework import *
from seven_cloudapp_frame.models.seven_model import InvokeResultData
from seven_cloudapp_frame.libs.customize.seven_helper import *
from seven_cloudapp_frame.models.enum import PlatType
from seven_cloudapp_frame.libs.common import *
from seven_cloudapp_frame.models.app_base_model import AppBaseModel
from seven_cloudapp_frame.models.mp_base_model import MPBaseModel
import re

class SafeHelper:
    """
    :description:安全帮助类
    """

    #sql关键字
    _sql_pattern_key = r"\b(and|like|exec|execute|insert|create|select|drop|grant|alter|delete|update|asc|count|chr|mid|limit|union|substring|declare|master|truncate|char|delclare|or)\b|(\*|;)"
    #Url攻击正则
    _url_attack_key = r"\b(alert|xp_cmdshell|xp_|sp_|restore|backup|administrators|localgroup)\b"

    @classmethod
    def get_redis_config(self, config_dict=None):
        """
        :Description: redis初始化
        :param config_dict: config_dict
        :return: redis_init
        :last_editors: HuangJianYi
        """
        config_dict = config.get_value("redis_safe")
        if not config_dict:
            config_dict = config.get_value("redis")
        if config_dict and config_dict.get("is_cluster", False):
            self.is_cluster = True
        else:
            self.is_cluster = False
        return config_dict

    @classmethod
    def authenticat_app_id(self, old_value, new_value):
        """
        :description: app_id鉴权
        :param old_value:旧值
        :param new_value:新值
        :return: 是否鉴权成功 True-成功 False-失败
        :last_editors: HuangJianYi
        """
        safe_config = share_config.get_value("safe_config", {})
        is_authenticat_app_id = safe_config.get("is_authenticat_app_id", True)
        if is_authenticat_app_id == True and old_value != new_value:
            return False
        else:
            return True

    @classmethod
    def is_contain_sql(self, str):
        """
        :description: 是否包含sql关键字
        :param str:参数值
        :return:
        :last_editors: HuangJianYi
        """
        result = re.search(self._sql_pattern_key, str.lower())
        if result:
            return True
        else:
            return False

    @classmethod
    def is_attack(self, str):
        """
        :description: 是否攻击请求
        :param str:当前请求地址
        :return:True是 False否
        :last_editors: HuangJianYi
        """
        if ":" in str:
            return True
        result = re.search(self._url_attack_key, str.lower())
        if result:
            return True
        else:
            return False

    @classmethod
    def filter_routine_key(self, key):
        """
        :description: 过滤常规字符
        :param key:参数值
        :return:
        :last_editors: HuangJianYi
        """
        routine_key_list = ["\u200b"]
        if not isinstance(key, str):
            return key
        for item in routine_key_list:
            key = key.replace(item, "")
        return key

    @classmethod
    def filter_sql(self, key):
        """
        :description: 过滤sql关键字
        :param key:参数值
        :return:
        :last_editors: HuangJianYi
        """
        if not isinstance(key, str):
            return key
        result = re.findall(self._sql_pattern_key, key.lower())
        for item in result:
            key = key.replace(item[0], "")
            key = key.replace(item[0].upper(), "")
        return key

    @classmethod
    def filter_special_key(self, key):
        """
        :description: 过滤sql特殊字符
        :param key:参数值
        :return:
        :last_editors: HuangJianYi
        """
        if not isinstance(key, str):
            return key
        special_key_list = ["\"", "\\", "/", "*", "'", "=", "-", "#", ";", "<", ">", "+", "%", "$", "(", ")", "%", "@","!"]
        for item in special_key_list:
            key = key.replace(item, "")
        return key

    @classmethod
    def check_attack_request(self, context):
        """
        :description: 校验是否攻击请求和是否包含非法参数值
        :param context:context
        :return:True满足条件直接拦截 False不拦截
        :last_editors: HuangJianYi
        """
        invoke_result_data = InvokeResultData()
        safe_config = share_config.get_value("safe_config",{})
        if safe_config.get("is_check_referer", False) == True:
            if not context.request.headers or not context.request.headers.get("Referer", ""):
                invoke_result_data.success = False
                invoke_result_data.error_code = "referer_error"
                invoke_result_data.error_message = f"请求不合法"
                return invoke_result_data
            authenticat_app_id = ""
            plat_type = share_config.get_value("plat_type", PlatType.wx.value)
            if plat_type == PlatType.wx.value:
                result = re.search(r"https://servicewechat.com/(\w+)/\d+", context.request.headers.get("Referer", ""))
                authenticat_app_id = result.group(1)
            elif plat_type == PlatType.jd.value:
                group_list = context.request.headers.get("Referer", "").split("/")
                authenticat_app_id = group_list[3] if len(group_list) >= 5 else ""
            if authenticat_app_id != share_config.get_value("app_id"):
                invoke_result_data.success = False
                invoke_result_data.error_code = "referer_error"
                invoke_result_data.error_message = f"请求不合法"
                return invoke_result_data

        is_filter_attack = safe_config.get("is_filter_attack", False)
        if is_filter_attack == True and context.request_params:
            for key in context.request_params.keys():
                if self.is_contain_sql(context.request_params[key]) or self.is_attack(context.request_params[key]):
                    invoke_result_data.success = False
                    invoke_result_data.error_code = "attack_error"
                    invoke_result_data.error_message = f"参数{key}:值不合法"
                    break
        return invoke_result_data

    @classmethod
    def check_params(self, must_params):
        """
        :description: 校验必传参数
        :return:InvokeResultData
        :last_editors: HuangJianYi
        """
        invoke_result_data = InvokeResultData()
        must_array = []
        if type(must_params) == str:
            must_array = must_params.split(",")
        if type(must_params) == list:
            must_array = must_params
        for must_param in must_array:
            if not must_param in self.request_params or self.request_params[must_param] == "":
                invoke_result_data.success = False
                invoke_result_data.error_code = "param_error"
                invoke_result_data.error_message = f"参数错误,缺少必传参数{must_param}"
                break
        return invoke_result_data

    @classmethod
    def check_handler_power(self, request_params, uri, handler_name, request_prefix="server"):
        """
        :description: 校验是否有权限访问接口
        :param request_params: 请求参数
        :param uri: 请求uri
        :param handler_name: handler_name
        :param request_prefix: 请求前缀(server或client)
        :return:True有False没有
        :last_editors: HuangJianYi
        """
        result = True
        if request_params and request_params.get("app_id", "") and f"/{request_prefix}/" in uri:
            server_power_config = config.get_value(f"{request_prefix}_power_config", {})
            power_codes = server_power_config.get(handler_name, "")
            if power_codes:
                app_base_model = AppBaseModel(context=self.context)
                app_info_dict = app_base_model.get_app_info_dict(request_params.get("app_id", ""))
                if app_info_dict:
                    mp_base_model = MPBaseModel(context=self.context)
                    result = mp_base_model.check_high_power(store_user_nick=app_info_dict["store_user_nick"], project_code=app_info_dict["project_code"], key_names=power_codes)
        return result

    @classmethod
    def check_current_limit_by_time_window(self, limit_name, limit_count, request_limit_time=1):
        """
        :description: 流量限流校验(采用时间窗口滑动算法)
        :param limit_name: 限量名称
        :param limit_count: 限制数
        :param request_limit_time: 限制时间，单位秒
        :return:是否进行限流 True-是 False-否
        :last_editors: HuangJianYi
        """
        try:
            safe_config = share_config.get_value("safe_config", {})
            project_name = config.get_value('project_name', '') if not safe_config.get("current_project_name") else safe_config.get("current_project_name")
            redis_config = self.get_redis_config()
            redis_init = RedisExHelper.init(config_dict=redis_config, decode_responses=True)
            current_time = int(time.time())
            window_start = current_time // request_limit_time * request_limit_time
            window_start = f"{limit_name}:{window_start}:db_{project_name}"
            request_count = redis_init.incr(window_start, 1)
            # 如果当前请求数超过限制数，则返回 True，表示需要限流
            if request_count > limit_count:
                return True
            redis_init.expire(window_start, request_limit_time * 2 )
            return False
        except Exception as ex:
            return False

    @classmethod
    def check_current_limit(self, app_id, current_limit_count, handler_name='', object_id=''):
        """
        :description: 流量限流校验
        :param app_id: 应用标识
        :param current_limit_count: 流量限制数
        :param handler_name: handler名字
        :param object_id: 用户唯一标识
        :return: True代表满足限制条件进行拦截
        :last_editors: HuangJianYi
        """
        try:
            if not app_id:
                app_id = 'global'
            safe_config = share_config.get_value("safe_config", {})
            project_name = config.get_value('project_name', '') if not safe_config.get("current_project_name") else safe_config.get("current_project_name")
            cache_key = f"request_user_list_{app_id}:{str(SevenHelper.get_now_int(fmt='%Y%m%d%H%M'))}"
            if handler_name:
                cache_key = f"{cache_key}:{handler_name}"
            else:
                current_limit_count = 0
            cache_key =  f"{cache_key}:db_{project_name}"
            redis_config = self.get_redis_config()
            redis_init = RedisExHelper.init(config_dict=redis_config, decode_responses=True)
            if current_limit_count == 0: # 如果没有传入流量限制数，则从配置中获取
                current_limit_count = safe_config.get("current_limit_user_count", 500) # 默认流量限制数量
                if safe_config.get("is_current_white", False) is True: # 是否开启白名单
                    count = redis_init.get(f"current_limit_white_list:{app_id}:db_{project_name}")
                    if count and int(count) > 0:
                        current_limit_count = int(count)
            if current_limit_count == 0:
                return False
            if object_id and redis_init.sismember(cache_key, object_id) == 1:
                return False
            if int(redis_init.scard(cache_key)) >= current_limit_count:
                return True
            else:
                return False
        except Exception as ex:
            return False

    @classmethod
    def add_current_limit_count(self, app_id, object_id, current_limit_count, handler_name=''):
        """
        :description: 流量UV限流计数
        :param app_id: 应用标识
        :param object_id: 用户唯一标识
        :param current_limit_count: 流量限制数
        :param handler_name: handler名字
        :return:
        :last_editors: HuangJianYi
        """
        if not object_id:
            return
        if not app_id:
            app_id = 'global'
        redis_config = self.get_redis_config()
        cache_key = f"request_user_list_{app_id}:{str(SevenHelper.get_now_int(fmt='%Y%m%d%H%M'))}"
        next_cache_key = f"request_user_list_{app_id}:{str(int((datetime.datetime.now() + datetime.timedelta(minutes=1)).strftime('%Y%m%d%H%M')))}"
        if handler_name:
            cache_key = f"{cache_key}:{handler_name}"
            next_cache_key = f"{next_cache_key}:{handler_name}"
        safe_config = share_config.get_value("safe_config", {})
        project_name = config.get_value('project_name', '') if not safe_config.get("current_project_name") else safe_config.get("current_project_name")
        cache_key = f"{cache_key}:db_{project_name}"
        next_cache_key = f"{next_cache_key}:db_{project_name}"
        redis_init = RedisExHelper.init(config_dict=redis_config, decode_responses=True)
        if current_limit_count == 0:
            current_limit_count = safe_config.get("current_limit_user_count", 500) # 默认流量限制数量
            if safe_config.get("is_current_white", False) is True: # 是否开启白名单
                count = redis_init.get(f"current_limit_white_list:{app_id}:db_{project_name}")
                if count and int(count) > 0:
                    current_limit_count = int(count)
        if int(redis_init.scard(cache_key)) < current_limit_count: # 达到限制数，则不能添加
            if self.is_cluster == False:
                pipeline = redis_init.pipeline()
                pipeline.sadd(cache_key, object_id)
                pipeline.expire(cache_key, 120)
                pipeline.sadd(next_cache_key, object_id)
                pipeline.expire(next_cache_key, 120)
                pipeline.execute()
            else:
                redis_init.sadd(cache_key, object_id)
                redis_init.expire(cache_key, 120)
                redis_init.sadd(next_cache_key, object_id)
                redis_init.expire(next_cache_key, 120)

    @classmethod
    def operate_current_limit_white_list(self, app_id, current_limit_count, operate_expire_time, operate_type=1):
        """
        :description: 增加UV限流商家白名单
        :param app_id: 应用标识
        :param object_id: 用户唯一标识
        :param current_limit_count: 流量限制数
        :param operate_type: 操作类型 1-添加 2-删除
        :return:
        :last_editors: HuangJianYi
        """
        safe_config = share_config.get_value("safe_config", {})
        project_name = config.get_value('project_name', '') if not safe_config.get("current_project_name") else safe_config.get("current_project_name")
        redis_config = self.get_redis_config()
        redis_init = RedisExHelper.init(config_dict=redis_config, decode_responses=True)
        key_name = f"current_limit_white_list:{app_id}:db_{project_name}"
        if operate_type == 1:
            cur_expire_time = redis_init.ttl(key_name)
            if not cur_expire_time:
                cur_expire_time = 0
            cur_expire_time = int(cur_expire_time) + int(operate_expire_time)
            redis_init.set(key_name, str(current_limit_count), ex=cur_expire_time)
        else:
            redis_init.delete(key_name)

    @classmethod
    def desensitize_word(self, word):
        """
        :description: 脱敏文字,变成*号
        :param word 文字
        :return:
        :last_editors: HuangJianYi
        """
        if len(word) == 0:
            return ""
        elif len(word) == 1:
            return "*"
        elif len(word) == 2:
            return word[0:1] + "*"
        elif len(word) == 3:
            return word[0:1] + "*" + word[2:3]
        else:
            num = 3 if len(word) - 3 > 3 else len(word) - 3
            star = '*' * num
            return word[0:2] + star + word[len(word) - 1:len(word)]

    @classmethod
    def desensitize_word_v2(self, word, head_chars=1, tail_chars=1):
        """
        :description: 自定义脱敏文字,变成*号
        :param word: 文字
        :param head_chars: 头部保留的字符数，默认为1
        :param tail_chars: 尾部保留的字符数，默认为1
        :return:
        :last_editors: HuangJianYi
        """
        from math import ceil

        if len(word) <= head_chars + tail_chars:
            if len(word) == 1:
                return '*'  # 对于长度为1的字符串显示为单个星号
            elif len(word) == 2:
                return word[0] + '*'  # 对于长度为2的字符串显示为首字符加一个星号
            else:
                return word  # 当长度不足但大于等于2时直接返回原文

        num = ceil((len(word) - head_chars - tail_chars) * 2 / 3)
        star = '*' * num
        return word[:head_chars] + star + word[-tail_chars:]



# -*- coding: utf-8 -*-
"""
:Author: HuangJingCan
:Date: 2022-12-15 10:52:33
:LastEditTime: 2022-12-15 13:58:26
:LastEditors: HuangJingCan
:Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class FunctionProductModel(CacheModel):
    def __init__(self, db_connect_key='db_middler_platform', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(FunctionProductModel, self).__init__(FunctionProduct, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类


class FunctionProduct:
    def __init__(self):
        super(FunctionProduct, self).__init__()
        self.app_id = 0  # 程序标识
        self.app_code = ""  # 程序代码
        self.app_name = ""  # 程序名称
        self.parent_id = 0  # 父标识
        self.name_path = ""  # 程序名路径
        self.id_path = ""  # 父路经
        self.sort_index = 0  # 排序
        self.create_date = "1900-01-01 00:00:00"  # 创建时间

    @classmethod
    def get_field_list(self):
        return ['app_id', 'app_code', 'app_name', 'parent_id', 'name_path', 'id_path', 'sort_index', 'create_date']

    @classmethod
    def get_primary_key(self):
        return "app_id"

    def __str__(self):
        return "function_product_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-07-26 09:40:36
@LastEditTime: 2022-12-07 14:18:05
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class CounterConfigModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(CounterConfigModel, self).__init__(CounterConfig, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类


class CounterConfig:

    def __init__(self):
        super(CounterConfig, self).__init__()
        self.id = 0  # id
        self.key_name = ""  # 计数key
        self.key_expire = 0  # 计数key过期时间
        self.db_connect_key = ""  # 数据库连接串Key
        self.table_name = ""  # 表名
        self.id_field_name = ""  # 主键字段名称
        self.value_field_name = ""  # 值字段名称
        self.is_release = 0  # 是否发布(1是0否)
        self.create_date = "1900-01-01 00:00:00"  # 创建时间

    @classmethod
    def get_field_list(self):
        return ['id', 'key_name', 'key_expire', 'db_connect_key', 'table_name', 'id_field_name', 'value_field_name', 'is_release', 'create_date']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "counter_config_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-07-20 10:09:52
@LastEditTime: 2023-12-25 14:48:41
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class AppTemplateModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(AppTemplateModel, self).__init__(AppTemplate, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类

class AppTemplate:

    def __init__(self):
        super(AppTemplate, self).__init__()
        self.id = 0  # id
        self.template_id = ""  # 模板标识
        self.product_name = "" # 产品名称
        self.product_icon = "" # 产品图标
        self.product_desc = "" # 产品简介
        self.client_ver = "" # 客户端版本号
        self.update_function = ""  # 版本更新内容
        self.create_date = "1900-01-01 00:00:00"  # 创建时间
        self.modify_date = "1900-01-01 00:00:00"  # 修改时间

    @classmethod
    def get_field_list(self):
        return ['id', "template_id", 'product_name', 'product_icon', 'product_desc', 'client_ver', 'update_function', "create_date", "modify_date"]

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "app_template_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-07-27 10:57:24
@LastEditTime: 2021-07-29 18:37:28
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class TaoLoginLogModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(TaoLoginLogModel, self).__init__(TaoLoginLog, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类

class TaoLoginLog:

    def __init__(self):
        super(TaoLoginLog, self).__init__()
        self.id = 0  # id
        self.open_id = ""  # open_id
        self.user_nick = ""  # 用户昵称
        self.store_user_nick = ""  # 主帐号名称（会员名）
        self.is_master = 0  # 是否主账号（1是0否）
        self.request_params = ""  # 参数
        self.create_date = "1900-01-01 00:00:00"  # 创建时间
        self.modify_date = "1900-01-01 00:00:00"  # 更新时间

    @classmethod
    def get_field_list(self):
        return ['id', 'open_id', 'user_nick', 'store_user_nick', 'is_master', 'request_params', 'create_date', 'modify_date']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "tao_login_log_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2022-04-24 15:15:19
@LastEditTime: 2025-05-30 19:05:52
@LastEditors: HuangJianYi
@Description: 
"""

from seven_framework.web_tornado.base_handler.base_api_handler import *
from seven_cloudapp_frame.libs.customize.seven_helper import *
from seven_cloudapp_frame.libs.customize.cryptography_helper import *
from seven_cloudapp_frame.libs.customize.safe_helper import *
from seven_cloudapp_frame.models.app_base_model import *
from urllib.parse import parse_qs


def filter_check_sign(sign_key, sign_lower=False, reverse=False, is_sign_key=False, expired_seconds=300, exclude_params=None):
    """
    :description: http请求签名验证装饰器
    :param sign_key: 参与签名的私钥
    :param sign_lower: 返回签名是否小写(默认大写)
    :param reverse: 是否反排序 False:升序 True:降序
    :param is_sign_key: 参数名是否参与签名(默认False不参与)
    :param expired_seconds: 接口timestamp过期时间(秒)
    :param exclude_params: 不参与签名的参数,支持list,str(英文逗号分隔)
    :last_editors: ChenXiaolei
    """
    def check_sign(handler):
        def wrapper(self, *args):
            # 签名参数
            sign_params = {}
            # 获取排除不需要签名的字段
            exclude_array = []
            if type(exclude_params) == str:
                exclude_array = exclude_params.split(",")
            if type(exclude_params) == list:
                exclude_array = exclude_params
            # 获取签名参数
            if not hasattr(self, "request_params"):
                if "Content-Type" in self.request.headers and self.request.headers[
                        "Content-type"].lower().find(
                            "application/json") >= 0 and self.request.body:
                    json_params = {}
                    try:
                        json_params = json.loads(self.request.body)
                    except:
                        self.response_json_error_params()
                        return
                    if json_params:
                        for field in json_params:
                            sign_params[field] = json_params[field]
                if self.request.arguments and len(self.request.arguments)>0:
                    for field in self.request.arguments:
                        sign_params[field] = self.get_param(field)
            else:
                sign_params = self.request_params

            if not sign_params or len(sign_params) < 2 or "timestamp" not in sign_params or "sign" not in sign_params:
                self.response_json_error_params("sign params error!")
                return

            sign_timestamp = int(sign_params["timestamp"])

            if expired_seconds and (not sign_timestamp or TimeHelper.add_seconds_by_timestamp(sign_timestamp, expired_seconds) < TimeHelper.get_now_timestamp() or sign_timestamp > TimeHelper.add_seconds_by_timestamp(second=expired_seconds)):
                self.response_json_error("error", "请求已失效.")
                return

            # 排除签名参数
            if exclude_array:
                for exclude_key in exclude_array:
                    if exclude_key in sign_params:
                        del sign_params[exclude_key]
            # 构建签名
            build_sign = SignHelper.params_sign_md5(
                sign_params, sign_key, sign_lower, reverse, is_sign_key)

            if not build_sign or build_sign != sign_params["sign"]:
                print(
                    f"http请求验签不匹配,收到sign:{sign_params['sign']},构建sign:{build_sign} 加密明文信息:{SignHelper.get_sign_params_str(sign_params,sign_key,reverse,is_sign_key)}")
                self.response_json_error("error", "sign error!")
                return

            return handler(self, *args)

        return wrapper

    return check_sign


def filter_check_params(must_params=None, check_user_code=False):
    """
    :description: 参数过滤装饰器 仅限handler使用,
                  提供参数的检查及获取参数功能
                  装饰器使用方法:
                  @client_filter_check_params("param_a,param_b,param_c")  或
                  @client_filter_check_params(["param_a","param_b,param_c"])
                  参数获取方法:
                  self.request_params[param_key]
    :param must_params: 必须传递的参数集合
    :param check_user_code: 是否校验用户标识必传
    :last_editors: HuangJianYi
    """
    def check_params(handler):
        def wrapper(self, **args):
            finally_must_params = must_params
            if hasattr(self, "must_params"):
                finally_must_params = self.must_params
            if type(finally_must_params) == str:
                must_array = finally_must_params.split(",")
            if type(finally_must_params) == list:
                must_array = finally_must_params

            if finally_must_params:
                for must_param in must_array:
                    if not must_param in self.request_params or self.request_params[must_param] == "":
                        self.response_json_error("param_error", f"参数错误,缺少必传参数{must_param}")
                        return
            if check_user_code == True and not self.get_user_id():
                self.response_json_error("param_error", f"参数错误,缺少必传参数user_code")
                return

            return handler(self, **args)

        return wrapper

    return check_params


def filter_check_current_limit(handler_name=None, current_limit_count=0, limit_params_dict={}):
    """
    :description: 流量限制过滤装饰器(UV) 仅限handler使用
    :param handler_name: handler名字
    :param current_limit_count: 流量限制数量
    :last_editors: HuangJianYi
    """
    def check_current(handler):
        def wrapper(self, **args):
            # 是否流量控制
            safe_config = share_config.get_value("safe_config", {})
            if safe_config.get("is_current_control", False) is True:  # 是否开启流量控制 0-关闭 1-开启
                app_id = self.get_app_id()
                object_id = self.get_open_id()
                if not object_id:
                    object_id = self.get_user_id()
                if SafeHelper.check_current_limit(app_id, current_limit_count, handler_name=handler_name, object_id=object_id) is True:
                    self.response_json_error("current_limit", "当前人数过多,请稍后再试")
                    return
                if handler_name:
                    SafeHelper.add_current_limit_count(app_id, object_id, current_limit_count, handler_name)

            return handler(self, **args)

        return wrapper

    return check_current


def filter_check_flow_limit(handler_name=None, flow_limit_api_count=0):
    """
    :description: 流量限制过滤装饰器(次数) 仅限handler使用
    :param handler_name: handler名字
    :param flow_limit_api_count: 流量限制数量
    :last_editors: HuangJianYi
    """
    def check_current(handler):
        def wrapper(self, **args):
            # 是否流量控制
            safe_config = share_config.get_value("safe_config", {})
            if safe_config.get("is_current_control", False) is True:  # 是否开启流量控制 0-关闭 1-开启
                app_id = self.get_app_id()
                limit_name = handler_name if handler_name else self.__class__.__name__
                limit_name += ":" + app_id
                limit_count = flow_limit_api_count if flow_limit_api_count else safe_config.get("flow_limit_api_count", 1000)
                if limit_count > 0:
                    if SafeHelper.check_current_limit_by_time_window(limit_name, limit_count, 1) is True:
                        self.response_json_error("current_limit", "当前人数过多,请稍后再试~")
                        return

            return handler(self, **args)

        return wrapper

    return check_current



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2022-03-03 11:04:55
@LastEditTime: 2025-08-18 19:16:48
@LastEditors: HuangJianYi
@Description: 兼容旧版openpyxl的Excel操作工具类（无WriteOnlyWorksheet依赖）
"""

import openpyxl
import os
import re
import xlwt
import xlrd
from xlutils.copy import copy
from typing import List, Dict, Optional, Union


class ExcelExHelper(object):
    """
    对Excel基本操作进行封装的类，兼容旧版openpyxl，优化了数据导出性能
    """

    @classmethod
    def __get_filename_by_xlsx(cls) -> str:
        return os.path.abspath('.') + '/export.xlsx'

    @classmethod
    def __get_filename_by_xls(cls) -> str:
        return os.path.abspath('.') + '/export.xls'

    @classmethod
    def __error_file_type(cls) -> None:
        raise TypeError("文件类型错误，仅支持xls和xlsx格式")

    @classmethod
    def judge_format(cls, filename: str) -> str:
        """判断文件格式（xls或xlsx）"""
        file_type = re.search(r'xls[x]?', filename)
        if file_type:
            return file_type.group()
        cls.__error_file_type()
        return ""  # 仅为类型提示兼容，实际不会执行

    @classmethod
    def create_file(cls, filename: str = '') -> None:
        """创建Excel文件（自动根据文件名后缀选择格式）"""
        if not filename:
            filename = cls.__get_filename_by_xlsx()

        file_type = cls.judge_format(filename)
        if file_type == 'xls':
            cls._create_file_by_xls(filename)
        elif file_type == 'xlsx':
            cls._create_file_by_xlsx(filename)
        else:
            cls.__error_file_type()

    @classmethod
    def input(cls, filename: str, sheet_name: str = 'Sheet') -> List[List[Union[str, int, float]]]:
        """从Excel导入数据（保持原功能）"""
        max_row = cls.get_max_row(filename, sheet_name)
        max_col = cls.get_max_column(filename, sheet_name)
        return cls.read_cell([1, 1], [max_row, max_col], filename, sheet_name)

    @classmethod
    def export(cls, data: List[Dict], filename: str = '', sheet_name: str = 'Sheet',
               use_pandas: bool = False) -> None:
        """
        数据导出到Excel（兼容旧版openpyxl）
        :param data: 字典数组
        :param filename: 文件名（包含路径）
        :param sheet_name: 工作表名称
        :param use_pandas: 是否使用pandas加速（需安装pandas和openpyxl）
        """
        if not data:
            return  # 空数据直接返回

        # 处理文件名
        if not filename:
            filename = cls.__get_filename_by_xlsx()
        file_type = cls.judge_format(filename)
        if file_type == 'xls':
            filename = filename.replace('.xls', '.xlsx')  # 统一用xlsx

        # 提取表头（优化：一次遍历收集所有唯一键）
        headers = []
        header_set = set()
        for item in data:
            for key in item.keys():
                if key not in header_set:
                    header_set.add(key)
                    headers.append(key)
        header_count = len(headers)

        # 选择导出方式
        if use_pandas:
            cls._export_with_pandas(data, headers, filename, sheet_name)
        else:
            # 直接写入（兼容模式：不依赖WriteOnlyWorksheet）
            cls._create_file_by_xlsx(filename)  # 确保文件存在
            cls._batch_write_xlsx_compatible(
                filename=filename,
                sheet_name=sheet_name,
                headers=headers,
                data=data
            )

    @classmethod
    def _export_with_pandas(cls, data: List[Dict], headers: List[str],
                            filename: str, sheet_name: str) -> None:
        """使用pandas批量导出（高效处理大数据）"""
        try:
            import pandas as pd
        except ImportError:
            raise ImportError("请安装pandas: pip install pandas openpyxl")

        # 转换数据为DataFrame（自动对齐表头）
        df = pd.DataFrame(data, columns=headers)
        # 写入Excel（使用openpyxl引擎）
        with pd.ExcelWriter(filename, engine='openpyxl', mode='a' if os.path.exists(filename) else 'w') as writer:
            df.to_excel(writer, sheet_name=sheet_name, index=False)

    @classmethod
    def _batch_write_xlsx_compatible(cls, filename: str, sheet_name: str,
                                    headers: List[str], data: List[Dict]) -> None:
        """兼容旧版openpyxl的批量写入方法（不依赖WriteOnlyWorksheet）"""
        # 检查文件是否存在，存在则加载，否则创建
        if os.path.exists(filename):
            wb = openpyxl.load_workbook(filename)
            # 如果工作表已存在，先删除（避免数据重复）
            if sheet_name in wb.sheetnames:
                del wb[sheet_name]
            ws = wb.create_sheet(sheet_name)
        else:
            wb = openpyxl.Workbook()
            # 删除默认工作表（如果存在）
            if 'Sheet' in wb.sheetnames:
                del wb['Sheet']
            ws = wb.create_sheet(sheet_name)

        # 写入表头
        for col, header in enumerate(headers, 1):  # 列从1开始
            ws.cell(row=1, column=col, value=header)

        # 批量写入数据行（优化：按行直接写入，不构建完整二维数组）
        for row_idx, item in enumerate(data, 2):  # 行从2开始（表头占1行）
            for col_idx, header in enumerate(headers, 1):
                ws.cell(row=row_idx, column=col_idx, value=item.get(header, ''))

        # 保存文件
        wb.save(filename)

    @classmethod
    def updata_cell(cls, row: int, column: int, value: Union[str, int, float],
                   filename: str = '', sheet_name: str = 'Sheet') -> None:
        """单个单元格更新"""
        cls.updata_cells([[value]], [row, column], filename, sheet_name)

    @classmethod
    def updata_cells(cls, data: List[List], cell_num_begin: List[int],
                    filename: str = '', sheet_name: str = 'Sheet') -> None:
        """更新单元格数据（优化批量写入）"""
        if not filename:
            filename = cls.__get_filename_by_xlsx()
            cls.create_file(filename)

        file_type = cls.judge_format(filename)
        if file_type == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            cls._updata_cell_by_xls(data, cell_num_begin, filename, sheet_name)
        elif file_type == 'xlsx':
            cls._updata_cell_by_xlsx(data, cell_num_begin, filename, sheet_name)
        else:
            cls.__error_file_type()

    @classmethod
    def _updata_cell_by_xlsx(cls, data: List[List], cell_num_begin: List[int],
                            filename: str, sheet_name: str) -> None:
        """xlsx批量更新（优化：减少循环嵌套）"""
        wb = openpyxl.load_workbook(filename)
        ws = wb[sheet_name] if sheet_name in wb.sheetnames else wb.create_sheet(sheet_name)

        start_row, start_col = cell_num_begin
        # 批量写入（直接按行索引操作）
        for i, row_data in enumerate(data):
            for j, value in enumerate(row_data):
                ws.cell(row=start_row + i, column=start_col + j, value=value)

        wb.save(filename)

    # 以下方法保持兼容优化
    @classmethod
    def read_cell(cls, cell_num_begin: List[int], cell_num_end: List[int],
                 filename: str = '', sheet_name: str = 'Sheet') -> List[List]:
        if not filename:
            filename = cls.__get_filename_by_xlsx()

        file_type = cls.judge_format(filename)
        if file_type == 'xls':
            return cls._read_cell_by_xls(cell_num_begin, cell_num_end, filename, sheet_name)
        elif file_type == 'xlsx':
            return cls._read_cell_by_xlsx(cell_num_begin, cell_num_end, filename, sheet_name)
        else:
            cls.__error_file_type()
            return []

    @classmethod
    def get_max_row(cls, filename: str, sheet_name: str = 'Sheet') -> int:
        if cls.judge_format(filename) == 'xls':
            sheet_name = sheet_name if sheet_name != 'Sheet' else 'Sheet1'
            return xlrd.open_workbook(filename).sheet_by_name(sheet_name).nrows
        else:
            return openpyxl.load_workbook(filename)[sheet_name].max_row

    @classmethod
    def get_max_column(cls, filename: str, sheet_name: str = 'Sheet') -> int:
        if cls.judge_format(filename) == 'xls':
            sheet_name = sheet_name if sheet_name != 'Sheet' else 'Sheet1'
            return xlrd.open_workbook(filename).sheet_by_name(sheet_name).ncols
        else:
            return openpyxl.load_workbook(filename)[sheet_name].max_column

    @classmethod
    def create_sheet(cls, sheet_name: str, filename: str = '') -> None:
        """修正原代码拼写错误"""
        if not filename:
            filename = cls.__get_filename_by_xlsx()
            cls.create_file(filename)

        if cls.judge_format(filename) == 'xlsx':
            wb = openpyxl.load_workbook(filename)
            if sheet_name not in wb.sheetnames:
                wb.create_sheet(sheet_name)
                wb.save(filename)
        else:
            print('仅支持xlsx创建表单')
            cls.__error_file_type()

    # 其他方法保持不变（省略重复代码，与上一版本一致）
    @classmethod
    def delete_sheet(cls, sheet_name: str, filename: str) -> None:
        if cls.judge_format(filename) == 'xlsx':
            wb = openpyxl.load_workbook(filename)
            if sheet_name in wb.sheetnames:
                del wb[sheet_name]
                wb.save(filename)
        else:
            print('仅支持xlsx删除表单')
            cls.__error_file_type()

    @classmethod
    def _create_file_by_xls(cls, filename: str = '') -> None:
        if not filename:
            filename = cls.__get_filename_by_xls()
        if cls.judge_format(filename) == 'xls':
            wb = xlwt.Workbook()
            wb.add_sheet('Sheet1')
            wb.save(filename)
        else:
            cls.__error_file_type()

    @classmethod
    def _create_file_by_xlsx(cls, filename: str = '') -> None:
        if not filename:
            filename = cls.__get_filename_by_xlsx()
        if cls.judge_format(filename) == 'xlsx':
            wb = openpyxl.Workbook()
            wb.save(filename)
        else:
            cls.__error_file_type()

    @classmethod
    def _updata_cell_by_xls(cls, data: List[List], cell_num_begin: List[int],
                           filename: str, sheet_name: str) -> None:
        if not filename:
            filename = cls.__get_filename_by_xls()
            cls._create_file_by_xls(filename)

        if cls.judge_format(filename) == 'xls':
            rb = xlrd.open_workbook(filename)
            wb = copy(rb)
            ws = wb.get_sheet(sheet_name) if isinstance(sheet_name, int) else wb.get_sheet(0)
            start_row, start_col = cell_num_begin
            for i, row_data in enumerate(data):
                for j, value in enumerate(row_data):
                    ws.write(start_row - 1 + i, start_col - 1 + j, value)
            wb.save(filename)
        else:
            cls.__error_file_type()

    @classmethod
    def _read_cell_by_xls(cls, cell_num_begin: List[int], cell_num_end: List[int],
                         filename: str, sheet_name: str) -> List[List]:
        if not filename:
            filename = cls.__get_filename_by_xls()
        if cls.judge_format(filename) == 'xls':
            sheet = xlrd.open_workbook(filename).sheet_by_name(sheet_name)
            data = []
            for i in range(cell_num_begin[0]-1, cell_num_end[0]):
                row = sheet.row_values(i, cell_num_begin[1]-1, cell_num_end[1])
                data.append(row)
            return data
        else:
            cls.__error_file_type()
            return []

    @classmethod
    def _read_cell_by_xlsx(cls, cell_num_begin: List[int], cell_num_end: List[int],
                          filename: str, sheet_name: str) -> List[List]:
        if not filename:
            filename = cls.__get_filename_by_xlsx()
        if cls.judge_format(filename) == 'xlsx':
            wb = openpyxl.load_workbook(filename, read_only=True)  # 只读模式优化
            ws = wb[sheet_name]
            data = []
            for row in ws.iter_rows(min_row=cell_num_begin[0], max_row=cell_num_end[0],
                                  min_col=cell_num_begin[1], max_col=cell_num_end[1],
                                  values_only=True):
                data.append(list(row))
            wb.close()
            return data
        else:
            cls.__error_file_type()
            return []

    @classmethod
    def _delete_cell_by_xls(cls, cell_num_begin: List[int], cell_num_end: List[int],
                           filename: str, sheet_name: str) -> None:
        empty_data = [['' for _ in range(cell_num_end[1]-cell_num_begin[1]+1)]
                     for _ in range(cell_num_end[0]-cell_num_begin[0]+1)]
        cls._updata_cell_by_xls(empty_data, cell_num_begin, filename, sheet_name)

    @classmethod
    def _delete_cell_by_xlsx(cls, cell_num_begin: List[int], cell_num_end: List[int],
                            filename: str, sheet_name: str) -> None:
        wb = openpyxl.load_workbook(filename)
        ws = wb[sheet_name]
        for row in range(cell_num_begin[0], cell_num_end[0]+1):
            for col in range(cell_num_begin[1], cell_num_end[1]+1):
                ws.cell(row=row, column=col, value='')
        wb.save(filename)



# -*- coding: utf-8 -*-
"""
:Author: HuangJingCan
:Date: 2022-12-15 10:52:33
@LastEditTime: 2023-04-07 16:09:03
@LastEditors: HuangJianYi
:Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class FunctionShopModel(CacheModel):
    def __init__(self, db_connect_key='db_middler_platform', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(FunctionShopModel, self).__init__(FunctionShop, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类


class FunctionShop:
    def __init__(self):
        super(FunctionShop, self).__init__()
        self.id = 0  # id
        self.product_id = 0  # 产品id
        self.cloud_app_id = 0  # 云应用id
        self.store_name = ""  # 店铺名称
        self.store_user_nick = ""  # 店铺主帐号名称（会员名）
        self.user_type = 0  # 用户类型：1-SAAS用户2-SAAS定制用户
        self.app_key = ""  # 商家实例化的app_key
        self.app_id = ""  # app_id
        self.trade_type = 0  # 行业类型（枚举型）
        self.service_date_start = "1900-01-01 00:00:00"  # 服务开始时间
        self.service_date_end = "1900-01-01 00:00:00"  # 服务结束时间
        self.skin_ids = ""  # 皮肤id列表（逗号隔开）
        self.function_ids = ""  # 功能id列表（逗号隔开）
        self.is_erp = 0  # 是否开启erp：0未开启1已开启
        self.erp_id = 0  # erp关联id（遁甲系统获取）
        self.erp_config_info = ""  # erp信息（json列表：{key:'字段名',value:'值'}）
        self.module_name = ""  # 模块名称
        self.module_pic = ""  # 模块图片
        self.is_release = 0  # 是否发布：0未发布1已发布
        self.release_date = "1900-01-01 00:00:00"  # 发布时间
        self.modify_date = "1900-01-01 00:00:00"  # 修改时间
        self.create_date = "1900-01-01 00:00:00"  # 创建时间

    @classmethod
    def get_field_list(self):
        return ['id', 'product_id', 'cloud_app_id', 'store_name', 'store_user_nick', 'user_type', 'app_key', 'app_id', 'trade_type', 'service_date_start', 'service_date_end', 'skin_ids', 'function_ids', 'is_erp', 'erp_id', 'erp_config_info', 'module_name', 'module_pic', 'is_release', 'release_date', 'modify_date', 'create_date']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "function_shop_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-09-13 16:26:10
@LastEditTime: 2025-06-30 19:11:43
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class QueueupLogModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(QueueupLogModel, self).__init__(QueueupLog, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类


class QueueupLog:
    def __init__(self):
        super(QueueupLog, self).__init__()
        self.id = 0  # id
        self.app_id = ""  # 应用标识
        self.act_id = 0  # 活动标识
        self.user_id = 0  # 用户标识
        self.open_id = ""  # open_id
        self.user_nick = ""  # 用户昵称
        self.queue_name = ""  # 队列名称
        self.queue_no = ""  # 排队号
        self.operate_type = 0  # 操作类型(1-加入队列 2-排到队列 3-参与抽赏 4-自动退出 5-手动退出)
        self.ip_name = ""  # 主题名称
        self.ip_id = 0  # 主题标识
        self.module_name = ""  # 模块名称
        self.module_id = 0  # 模块标识
        self.info_json = {}  # 信息json
        self.remain_time = 0  # 剩余时间(单位秒)
        self.create_date = "1900-01-01 00:00:00"  # 创建时间
        self.create_day = 0  # 创建时间天

    @classmethod
    def get_field_list(self):
        return ['id', 'app_id', 'act_id', 'user_id', 'open_id', 'user_nick', 'queue_name', 'queue_no', 'operate_type', 'ip_name', 'ip_id', 'module_name', 'module_id', 'info_json', 'remain_time', 'create_date', 'create_day']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "queueup_log_tb"



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-07-15 11:30:45
@LastEditTime: 2025-08-19 09:44:47
@LastEditors: HuangJianYi
@Description: 
"""
import hashlib
from seven_framework import *
from seven_cloudapp_frame.libs.common import *
from seven_cloudapp_frame.libs.customize.excel_helper import ExcelExHelper


class OSSHelper:
    """
    :description: 阿里云存储帮助类 上传文件、数据导入excel并返回下载地址
    :param {type} 
    :return: 
    :last_editors: HuangJianYi
    """
    logger_error = Logger.get_logger_by_name("log_error")

    @classmethod
    def get_oss_config(self):
        oss_config = share_config.get_value("oss_config", {})
        self.ak_id = oss_config.get("ak_id", "")
        self.ak_secret = oss_config.get("ak_secret", "")
        self.bucket_name = oss_config.get("bucket_name", "")
        self.end_point = oss_config.get("end_point", "")
        self.domain = oss_config.get("domain", "")
        if not self.domain:
            self.domain = oss_config.get("demain", "")
        self.folder = oss_config.get("folder", "")

    @classmethod
    def upload(self, file_name, local_file='', folder='', is_auto_name=True, data=None):
        """
        :description: 上传文件
        :param file_name：文件名称
        :param local_file：本地文件地址
        :param folder：本地文件地址
        :param is_auto_name：是否生成随机文件名
        :param data：需要上传的数据
        :return: 
        :last_editors: HuangJianYi
        """
        import oss2
        import os
        self.get_oss_config()
        # 文件名
        file_name = os.path.basename(local_file) if local_file != "" else file_name

        if is_auto_name:
            file_extension = os.path.splitext(file_name)[1]
            file_name = str(int(time.time())) + UUIDHelper.get_uuid() + file_extension

        auth = oss2.Auth(self.ak_id, self.ak_secret)
        bucket = oss2.Bucket(auth, self.end_point, self.bucket_name)
        if not folder:
            folder = self.folder
        folder = folder.strip('/')
        folder = folder + "/" if folder != "" else folder
        file_name = folder + file_name

        # 上传文件
        # 如果需要上传文件时设置文件存储类型与访问权限，请在put_object中设置相关headers, 参考如下。
        # headers = dict()
        # headers["x-oss-storage-class"] = "Standard"
        # headers["x-oss-object-acl"] = oss2.OBJECT_ACL_PRIVATE
        # result = bucket.put_object('<yourObjectName>', 'content of object', headers=headers)
        if local_file:
            result = bucket.put_object_from_file(file_name, local_file)
        else:
            result = bucket.put_object(file_name, data)

        resource_path = ''
        if result.status == 200:
            resource_path = self.domain + file_name
            # # HTTP返回码。
            # print('http status: {0}'.format(result.status))
            # # 请求ID。请求ID是请求的唯一标识，强烈建议在程序日志中添加此参数。
            # print('request_id: {0}'.format(result.request_id))
            # # ETag是put_object方法返回值特有的属性。
            # print('ETag: {0}'.format(result.etag))
            # # HTTP响应头部。
            # print('date: {0}'.format(result.headers['date']))

        return resource_path

    @classmethod
    def export_excel(self, import_data):
        """
        :description: 把数据导入excel并返回下载地址
        :param import_data:导入数据
        :return excel下载地址
        :last_editors: HuangJianYi
        """
        resource_path = ""
        if import_data:
            try:
                if not os.path.exists("temp"):
                    os.makedirs("temp")
                path = "temp/" + UUIDHelper.get_uuid() + ".xlsx"
                ExcelExHelper.export(import_data, path)
                resource_path = self.upload("", path, share_config.get_value("oss_folder"), False)
                os.remove(path)
            except Exception as ex:
                self.logger_error.error("【数据导入excel】" + traceback.format_exc())
        return resource_path


class COSHelper:
    """
    :description: 腾讯云存储帮助类 上传文件、数据导入excel并返回下载地址
    :param {type} 
    :return: 
    :last_editors: HuangJianYi
    """
    logger_error = Logger.get_logger_by_name("log_error")

    @classmethod
    def get_cos_config(self):
        oss_config = share_config.get_value("cos_config", {})
        self.access_key = oss_config.get("access_key", "")
        self.secret_key = oss_config.get("secret_key", "")
        self.bucket_name = oss_config.get("bucket_name", "")
        if not self.bucket_name:
            self.bucket_name = oss_config.get("bucket", "")
        self.end_point = oss_config.get("end_point", "")
        self.domain = oss_config.get("domain", "")
        self.folder = oss_config.get("folder", "")

    @classmethod
    def upload(self, file_name, data=None, is_auto_name=True):
        """
        :description:上传文件
        :param file_name：文件名称
        :param data：需要上传的数据
        :param is_auto_name: 是否生成随机文件名
        :return: 
        :last_editors: HuangJianYi
        """
        from seven_framework.file import COSHelper
        self.get_cos_config()
        if is_auto_name == True:
            file_extension = os.path.splitext(file_name)[1]
            file_name = str(int(time.time())) + UUIDHelper.get_uuid() + file_extension
        object_name = self.folder + "/" + file_name
        result = COSHelper(self.access_key, self.secret_key, self.end_point).put_file(self.bucket_name, object_name, data)
        if result == True:
            return self.domain + "/" + object_name
        else:
            return ""

    @classmethod
    def put_file_from_file_path(self, local_file=''):
        """
        :description:上传文件根据路径
        :param local_file：本地文件地址
        :return: 
        :last_editors: HuangJianYi
        """
        from seven_framework.file import COSHelper
        self.get_cos_config()
        file_name = os.path.basename(local_file)
        object_name = self.folder + "/" + file_name
        result = COSHelper(self.access_key, self.secret_key, self.end_point).put_file_from_file_path(self.bucket_name, object_name, local_file)
        if result == True:
            return self.domain + "/" + object_name
        else:
            return ""

    @classmethod
    def export_excel(self, import_data):
        """
        :description: 把数据导入excel并返回下载地址
        :param import_data:导入数据
        :return excel下载地址
        :last_editors: HuangJianYi
        """
        resource_path = ""
        if import_data:
            try:
                if not os.path.exists("temp"):
                    os.makedirs("temp")
                path = "temp/" + UUIDHelper.get_uuid() + ".xlsx"
                ExcelExHelper.export(import_data, path)
                resource_path = self.put_file_from_file_path(path)
                os.remove(path)
            except Exception as ex:
                self.logger_error.error("【数据导入excel】" + traceback.format_exc())
        return resource_path


class BOSHelper:
    """
    :description: 百度云帮助类
    """
    logger_error = Logger.get_logger_by_name("log_error")

    @classmethod
    def get_bos_client(self):
        """
        :description: 获取百度云存储对象的client客户端
        """
        from baidubce.auth.bce_credentials import BceCredentials
        from baidubce.bce_client_configuration import BceClientConfiguration
        from baidubce.services.bos.bos_client import BosClient

        bos_config = share_config.get_value("bos_config", {})
        access_key = bos_config.get("access_key", "")
        secret_key = bos_config.get("secret_key", "")
        end_point = bos_config.get("end_point", "")
        # 创建认证组
        credentials = BceCredentials(access_key_id=access_key, secret_access_key=secret_key)
        # 创建BceClientConfiguration
        config = BceClientConfiguration(credentials=credentials, endpoint=end_point)
        # 获取到客户端
        bos_client = BosClient(config=config)
        return bos_client

    @classmethod
    def context_md5(self, stream):
        md5 = hashlib.md5()
        md5.update(stream)
        content_md5 = base64.standard_b64encode(md5.digest())
        return content_md5

    @classmethod
    def put_object(self, file_name, stream):
        """
        :description:根据文件名上传到服务器
        :param file_name: 文件名
        :param stream: 二进制流
        :return: 
        :last_editors: HuangJianYi
        """
        from baidubce.services.bos import storage_class

        bos_client = self.get_bos_client()
        bos_config = share_config.get_value("bos_config", {})
        folder = bos_config.get("folder", "")
        bucket_name = bos_config.get("bucket_name", "")
        domain = bos_config.get("domain", "")
        object_key = folder + "/" + file_name
        # 根据文件名上传文件
        try:
            result = bos_client.put_object(bucket_name=bucket_name, key=object_key, data=stream, content_length=len(stream), content_md5=self.context_md5(stream), storage_class=storage_class.STANDARD)
            if result:
                return domain + "/" + object_key
        except Exception as e:
            self.logger_error.error("【上传文件出错】" + str(traceback.format_exc()))
        return ""

    @classmethod
    def put_object_from_file(self, file_name, local_file='', folder='', is_auto_name=True):
        """
        :description:根据文件名上传到服务器
        :param file_name: 文件名
        :param local_file: 本地文件地址
        :param folder: 文件夹
        :param is_auto_name: 是否生成随机文件名
        :return: 链接地址
        :last_editors: HuangJianYi
        """
        # 文件名
        file_name = os.path.basename(local_file) if local_file != "" else file_name
        if is_auto_name:
            file_extension = os.path.splitext(file_name)[1]
            file_name = UUIDHelper.get_uuid().replace("-", "") + file_extension
        bos_client = self.get_bos_client()
        bos_config = share_config.get_value("bos_config", {})
        if not folder:
            folder = bos_config.get("folder", "")
        bucket_name = bos_config.get("bucket_name", "")
        domain = bos_config.get("domain", "")
        object_key = folder + "/" + file_name
        try:
            result = bos_client.put_object_from_file(bucket_name=bucket_name, key=object_key, file_name=local_file)
            if result.status == 200:
                return domain + "/" + object_key
        except Exception as e:
            self.logger_error.error("【上传文件出错】" + str(traceback.format_exc()))
        return ""

    @classmethod
    def upload(self, file_name, is_auto_name=True, data=None):
        """
        :description:上传文件
        :param file_name：文件名称
        :param is_auto_name：是否生成随机文件名
        :param data：需要上传的数据
        :return: 
        :last_editors: HuangJianYi
        """
        if is_auto_name:
            file_extension = os.path.splitext(file_name)[1]
            file_name = str(int(time.time())) + UUIDHelper.get_uuid() + file_extension
        return self.put_object(file_name, data)

    @classmethod
    def export_excel(self, import_data):
        """
        :description: 把数据导入excel并返回下载地址
        :param import_data:导入数据
        :return excel下载地址
        :last_editors: HuangJianYi
        """
        resource_path = ""
        if import_data:
            try:
                if not os.path.exists("temp"):
                    os.makedirs("temp")
                path = "temp/" + UUIDHelper.get_uuid() + ".xlsx"
                ExcelExHelper.export(import_data, path)
                resource_path = self.put_object_from_file("", path, share_config.get_value("oss_folder"), False)
                os.remove(path)
            except Exception as ex:
                self.logger_error.error("【数据导入excel】" + traceback.format_exc())
        return resource_path



# -*- coding: utf-8 -*-
"""
@Author: HuangJianYi
@Date: 2021-08-03 15:45:17
@LastEditTime: 2021-08-03 15:45:58
@LastEditors: HuangJianYi
@Description: 
"""
#此文件由rigger自动生成
from seven_framework.mysql import MySQLHelper
from seven_framework.base_model import *
from seven_cloudapp_frame.models.cache_model import *


class MarketingProgramModel(CacheModel):
    def __init__(self, db_connect_key='db_cloudapp', sub_table=None, db_transaction=None, context=None, is_auto=False):
        super(MarketingProgramModel, self).__init__(MarketingProgram, sub_table)
        self.db = MySQLHelper(self.convert_db_config(db_connect_key, is_auto))
        self.db_connect_key = db_connect_key
        self.db_transaction = db_transaction
        self.db.context = context

    #方法扩展请继承此类

class MarketingProgram:

    def __init__(self):
        super(MarketingProgram, self).__init__()
        self.id = 0  # id
        self.link_title = ""  # 链接标题
        self.link_url = ""  # 链接地址
        self.content = ""  # 内容说明
        self.sort_index = 0  # 排序
        self.is_release = 0  # 是否发布（1是0否）
        self.release_date = "1900-01-01 00:00:00"  # 发布时间
        self.create_date = "1900-01-01 00:00:00"  # 创建时间
        self.modify_date = "1900-01-01 00:00:00"  # 更新时间

    @classmethod
    def get_field_list(self):
        return ['id', 'link_title', 'link_url', 'content', 'sort_index', 'is_release', 'release_date', 'create_date', 'modify_date']

    @classmethod
    def get_primary_key(self):
        return "id"

    def __str__(self):
        return "marketing_program_tb"
